CON
  _clkmode = 0
  _xinfreq = $4E495053 ' SPIN

  NEW_LINE = 10
  eeprom_pin = 28
  sdcard_pin = 16

OBJ
  c : "clib"
  mem : "cmalloc"
  ser : "cserial"
  eeprom : "pasm_i2c_driver"
  rendez : "rendezvous"

DAT
  badflag long 0
  buffer byte 0[100]
  argv long 0[20]
  strbuf byte 0[100]
  
DAT
  redirectin long 0
  redirectout long 0
  commands
  byte "dump", 0, "peek", 0, "poke", 0
  byte "dumpm", 0, "peekm", 0, "pokem", 0
  byte "stack", 0, "malloc", 0, "help", 0, "exit", 0, 0

PUB main | i, blocklongs, num, index, outfile, infile, argc, startaddr, endaddr, TopAddr, TopObjList
  c.enter

  REPEAT
    ' Close redirection files if opened
    if redirectin
      c.resetstdin
      c.fclose(redirectin)
      redirectin := 0
    if redirectout
      c.resetstdout
      c.fclose(redirectout)
      redirectout := 0

    c.printf0(STRING("\n>"))
    c.gets(@buffer)
    argc := tokenize(@buffer, @argv)
    argc := Redirection(argc)

    IF (argc == 0)
      NEXT

    num := 0
    index := @commands
    repeat while byte[index]
      if c.strcmp(argv[0], index) == 0
        quit
      index += strsize(index) + 1
      num++

    case num
      0 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        repeat i from 0 to num - 1
          if ((i & 15) == 0)
            c.printf1(STRING("\n%05x:"), i + index)
          c.printf1(STRING(" %02x"), eeprom.ReadByte(28, $A0, i + index))
        c.printf0(STRING("\n"))
      1 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.printf2(STRING("EEPROM[%04x] = %02x\n"), index, eeprom.ReadByte(28, $A0, index))
      2 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        eeprom.WriteByte(28, $A0, index, num)
        repeat while eeprom.WriteWait(28, $A0, index)
      3 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        DumpMemory(index, num)
      4 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.printf2(STRING("Memory[%04x] = %02x\n"), index, byte[index])
      5 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        byte[index] := num
      6:
        c.CheckStackSpace
      7:
        c.CheckMallocSpace
      8:
        if (infile := c.fopen(string("/bin/help.txt"), string("r")))
          repeat while c.fgets(@buffer, 100, infile)
            c.fputs(@buffer, long[rendez#stdout])
          c.fclose(infile)
      9:
        c.exit(@@0)
      other:
        i := c.run(argv[0], argc, @argv)
        if i < 0
          c.printf1(string("Could not run %s\n"), argv[0])
        else
          index := rendez#cogtable + (i * 24)
          repeat while byte[index]

PUB AddToCogTable(cognum, type, name, startaddr, size) | entryptr
  entryptr := rendez#cogtable + (cognum * 24)
  byte[entryptr] := type
  bytemove(entryptr+1, name, 19)
  byte[entryptr+19] := 0
  word[entryptr+20] := startaddr
  word[entryptr+22] := size

PUB RemoveFromCogTable(cognum) | entryptr
  entryptr := rendez#cogtable + (cognum * 24)
  byte[entryptr] := 0

PUB Redirection(argc) | i, argc1, ptr, char, mode
  if (argc < 2)
    return argc
  argc1 := 1
  repeat i from 1 to argc - 1
    ptr := argv[i]
    char := byte[ptr++]
    if char == "<"
      if (byte[ptr] == 0)
        ptr := argv[++i]
        if (i => argc)
          c.printf0(string("Redirection error\n"))
          argc1 := 0
          quit
      redirectin := c.fopen(ptr, string("r"))
      ifnot redirectin
        c.printf1(string("Could not open %s for input\n"), ptr)
        argc1 := 0
        quit
      else
        c.setstdin(redirectin)
    elseif char == ">"
      if (byte[ptr] == ">")
        ptr++
        mode := string("a")
      else
        mode := string("w")
      if (byte[ptr] == 0)
        ptr := argv[++i]
        if (i => argc)
          c.printf0(string("Redirection error\n"))
          argc1 := 0
          quit
      redirectout := c.fopen(ptr, mode)
      ifnot redirectout
        c.printf1(string("Could not open %s for output\n"), ptr)
        argc1 := 0
        quit
      else
        c.setstdout(redirectout)
    else
      argv[argc1++] := argv[i]
  return argc1

' This routine returns a pointer to the first character not
' matching the passed value
PUB SkipChar(str, value_0)
  REPEAT WHILE (BYTE[str])
    IF (BYTE[str] <> value_0)
      QUIT
    str++
  RETURN str
  
' This routine returns a pointer to the first charcter that
' matches the passed value
PUB FindChar(str, value_0)
  REPEAT WHILE (BYTE[str])
    IF (BYTE[str] == value_0)
      QUIT
    str++
  RETURN str
  
' This routine extracts tokens from the input string that
' are seperated by at least one space character
PUB tokenize(ptr, tokens) | num
  num := 0
  repeat
    ptr := SkipChar(ptr, " ")
    ifnot byte[ptr]
      quit
    num++
    if byte[ptr] == $22
      long[tokens] := ++ptr
      ptr := FindChar(ptr, $22)
    else
      long[tokens] := ptr
      ptr := FindChar(ptr, " ")
    tokens += 4
    IF (BYTE[ptr] == 0)
      QUIT
    BYTE[ptr++] := 0
  RETURN num
  
PUB DumpMemory(address, num) | i, ptr
  ptr := address
  i := 0
  REPEAT WHILE (i < num)
    IF ((i & 15) == 0)
      c.printf1(STRING("\n%05x:"), i + address)
    c.printf1(STRING(" %02x"), BYTE[ptr][i])
    i++
  c.printf0(STRING("\n"))

{
PUB run(argc, pargv) | infile, fname, size, AppAddr, i, sp, cognum
{
  if argc < 2
    c.printf0(string("usage: file [parms]\n"))
    return
  fname := long[pargv][1]
  c.printf1(string("run %s\n"), fname)
}
  fname := long[pargv][0]
  infile := c.fopen(fname, string("r"))
  ifnot infile
    c.printf1(string("Could not open %s\n"), fname)
    return
  ' Determine the file size
  c.fseek(infile, 0, c#SEEK_END)
  size := c.ftell(infile)
  c.fseek(infile, 0, c#SEEK_SET)
  ' Allocate the memory plus some for the stack
  AppAddr := c.calloc(size + 800)
  ifnot AppAddr
    c.printf1(string("Not enough memory to run %s\n"), fname)
    c.fclose(infile)
    return
  'c.memset(AppAddr, $a5, size + 400)
  c.fread(AppAddr, 1, size, infile)
  c.fclose(infile)
  ' Fixup
  Repeat i from 0 to 4
    Word[AppAddr+6+i<<1] += AppAddr
  sp := word[AppAddr+$a]
  long[sp+4] := argc
  long[sp+8] := pargv
  'c.memset(sp, $a5, 100)
  byte[AppAddr] := 1
  cognum := Cognew($F004, AppAddr + 4)

  AddToCogTable(cognum, 1, fname, AppAddr, size + 800)

  repeat while byte[AppAddr]

  'c.printf1(string("Stopping cog %d\n"), cognum)
  cogstop(cognum)

  c.free(AppAddr)

  RemoveFromCogTable(cognum)
}
