CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  eeprom_pin = 28
  'sdcard_pin = 16

  file_info_serial = 1
  file_info_fileio = 2
  
  file_info_type   = 0
  file_info_data   = 1
  file_info_size   = 8

OBJ
  rendez : "rendezvous"
  fileio : "cfileio0"
  mem : "cmalloc"
  ser : "cserial"
  serp : "cserialp"
  i2c_p : "pasm_i2c_driver_p"
  spi_p : "safe_spi_p"
  i2c : "pasm_i2c_driver"

DAT
  sdpin byte 0[4]
  pins byte "DO ", 0, "CLK", 0, "DI ", 0, "CS ", 0

PUB main | i, blocklongs, num, index, outfile, infile, argc, startaddr, endaddr, TopAddr, TopObjList, cognum
  longfill($7f00, 0, 256)
  start1(31, 30, %10000, 57600, 200)
  AddToCogTable(0, 1, string("bootsd.binary"), 0, long[rendez#memfreelist])
  AddToCogTable(1, 2, string("serial I/O"), 0, 0)
  cognum := i2c_p.Initialize(eeprom_pin)
  AddToCogTable(cognum - 1, 2, string("I2C Driver"), 0, 0)

  repeat
    ser.dbprintf0(string("Press ESC for setup or any other key to start\n"))
    if ((i := ser.rxtime(1000)) <> -1)
      quit

  if long[@sdpin] == 0 OR i == 27
    repeat index from 0 to 3
      ser.dbprintf1(string("Enter %s pin number: "), @pins + (index << 2))
      num := 0
      repeat
        i := ser.rx
        ser.tx(i)
        if i < "0" OR i > "9"
          quit
        num := (num * 10) + i - "0"
      sdpin[index] := num
    i2c.WriteLong(28, $A0, @sdpin, long[@sdpin])
    repeat while i2c.WriteWait(28, $A0, @sdpin)

  num := spi_p.start_explicit(sdpin[0], sdpin[1], sdpin[2], sdpin[3])
  AddToCogTable(cognum, 2, string("SPI Driver"), 0, 0)

  StartFileIO

  if fileio.mount(1, string("disk1"), sdpin) =< 0
    ser.tx("%")
    'ser.dbprintf0(STRING("\nmount failed.\n"))
    'ser.dbprintf0(STRING("Run install to initialize the file system\n"))
    repeat
  fileio.chdir(string("/"))

  fileio.run(string("shell"), 0, 0, 1)

  repeat

PUB AddToCogTable(cognum, type, name, startaddr, size) | entryptr
  entryptr := rendez#cogtable + (cognum * 24)
  byte[entryptr] := type
  bytemove(entryptr+1, name, 19)
  byte[entryptr+19] := 0
  word[entryptr+20] := startaddr
  word[entryptr+22] := size

DAT
  fileio1 FILE "cfileiosd1.binary"

PUB StartFileIO | AppAddr, size, i, cognum
  'ser.tx("&")
  'ser.dbprintf0(string("StartFileIO\n"))
  size := word[@fileio1 + $a] ' Start of stack
  ' Allocate the memory plus some for the stack
  AppAddr := mem.calloc(size + 800)
  ifnot AppAddr
    'ser.tx("#")
    ser.dbprintf0(string("Not enough memory to run file I/O\n"))
    return
  bytemove(AppAddr, @fileio1, size)
  ' Fixup
  Repeat i from 0 to 4
    Word[AppAddr+6+i<<1] += AppAddr
  cognum := Cognew($F004, AppAddr + 4)
  AddToCogTable(cognum, 1, string("cfileiosd1.binary"), AppAddr, size + 800)
  waitcnt(clkfreq/100+cnt)

PUB start1(rxpin, txpin, mode, baudrate, stacksize) | stdio, stdin, stdout
{{
  This routine calls the malloc initialization routine.  The number of longs in
  the top object stack is set by the value of "stacksize".  It also calls the
  openserial function to initialize the standard I/O serial port.  It returns the
  stdio pointer if successful, or NULL if not.
}}
  mem.mallocinit(stacksize)
  stdio := openserial(rxpin, txpin, mode, baudrate, 512, 64)
  stdin := mem.malloc(file_info_size)
  stdout := mem.malloc(file_info_size)
  long[rendez#stdio] := stdio
  long[rendez#stdin] := stdin
  long[rendez#stdout] := stdout
  bytemove(stdin, stdio, file_info_size)
  bytemove(stdout, stdio, file_info_size)
  return stdio

PUB openserial(rxpin, txpin, mode, baudrate, rxsize, txsize) | pfileinfo, file_data
{{
  This routine allocates memory for a file info and serial port data structure.
  It calls the start1 function in the cserial object to create a serial port in
  the next available cog.  It returns a pointer to the file info structure if
  successful, or a NULL value if not successful.
}}
  pfileinfo := mem.malloc(file_info_size + ser#header_size + rxsize + txsize)
  ifnot pfileinfo
    return 0
  file_data := pfileinfo + file_info_size
  'file_data := ser.gethandle1 
  long[pfileinfo][file_info_type] := file_info_serial
  long[pfileinfo][file_info_data] := file_data
  ifnot serp.start1(file_data, rxpin, txpin, mode, baudrate, rxsize, txsize)
    mem.free(pfileinfo)
    return 0
  return pfileinfo
